@@ -239,85 +239,96 @@ struct tree_descender
 			return;
 		}
 
-		const size_t w = std::min( repLength( it ), W );
+        // get bounds on g
+        const double min_g = _model._prior._psp->get_min_g( it );
 
-		MYRRH_ASSERT( w > previous_w || 0 == previous_w );
-		const size_t num_occurrences = countOccurrences( it );
-		const prefix_t pre = prefix( representative( it ), w );
+        // the depth we're at
+        const size_t w = std::min( repLength( it ), W );
+        MYRRH_ASSERT( w > previous_w || 0 == previous_w );
 
-		// update our partial evaluation of the background model
-		partial_bg_eval = _model.bg.update_partial_eval( pre, partial_bg_eval, previous_w );
+        // the number of possible binding sites
+        const size_t num_occurrences = countOccurrences( it );
 
-		// get a bound on the evaluation under the background model
-		const double bg_log_likelihood_bound = _model.bg.get_min_log_likelihood( it, partial_bg_eval, w, W );
+        // if g = 1. then we cannot have a binding site down this part of the suffix tree
+        if ( min_g >= 1. ) {
 
-		// get a bound on log(1-g)
-		const double weight_bound = _model._prior._psp->get_term_bound( it );
+            stats.discarded( w, num_occurrences ); // update stats
 
-		// the threshold on log(<Z>)
-		const double log_Z_threshold = static_cast< Derived * >( this )->get_log_Z_threshold();
+        } else {
 
-		// log lambda
-		const double log_lambda = _model._prior.lambda.get_log_lambda();
+            // the prefix represented by this node.
+            const prefix_t pre = prefix( representative( it ), w );
 
-		// get the bound on log p(X_n|\theta_BS)
-		const double eval_threshold =
-		    log_Z_threshold
-            + bg_log_likelihood_bound
-			- weight_bound
-			- log_lambda
-			;
+            // update our partial evaluation of the background model
+            partial_bg_eval = _model.bg.update_partial_eval( pre, partial_bg_eval, previous_w );
 
-		// we have to evaluate the W-mer and its reverse complement.
-		// start with the W-mer
-		if( partial_eval_pair.first ) { // do we have an eval on the positive strand?
-			partial_eval_pair.first =
-				_model.bs.template update_partial_eval_thresholded< false >(
-					pre,
-					*partial_eval_pair.first,
-					previous_w,
-					eval_threshold
-				);
-		}
+            // get a bound on the evaluation under the background model
+            const double bg_log_likelihood_bound = _model.bg.get_min_log_likelihood( it, partial_bg_eval, w, W );
 
-		// now evaluate the reverse complement
-		if( partial_eval_pair.second ) { // do we have an eval on the negative strand?
-			partial_eval_pair.second =
-				_model.bs.template update_partial_eval_thresholded< true >(
-					pre,
-					*partial_eval_pair.second,
-					previous_w,
-					eval_threshold
-				);
-		}
+            // the threshold on log likelihood ratio
+            const double llr_threshold = static_cast< Derived * >( this )->get_llr_threshold();
 
-		// do we have any evals left?
-		if( ! partial_eval_pair ) { // do we have an eval on either strand?
+            // log prior
+            const double log_prior = _model._prior.calculate_log_prior( min_g );
 
-			stats.discarded( w, num_occurrences ); // update stats
+            // get the bound on log p(X_n|\theta_BS)
+            const double eval_threshold =
+                llr_threshold
+                - log_prior
+                + bg_log_likelihood_bound
+                ;
 
-		} else {
+            // we have to evaluate the W-mer and its reverse complement.
+            // start with the W-mer
+            if( partial_eval_pair.first ) { // do we have an eval on the positive strand?
+                partial_eval_pair.first =
+                    _model.bs.template update_partial_eval_thresholded< false >(
+                        pre,
+                        *partial_eval_pair.first,
+                        previous_w,
+                        eval_threshold
+                    );
+            }
 
-			stats.evaluated( w, num_occurrences ); // update stats
+            // now evaluate the reverse complement
+            if( partial_eval_pair.second ) { // do we have an eval on the negative strand?
+                partial_eval_pair.second =
+                    _model.bs.template update_partial_eval_thresholded< true >(
+                        pre,
+                        *partial_eval_pair.second,
+                        previous_w,
+                        eval_threshold
+                    );
+            }
 
-			// did we get to the bottom?
-			if( W == w ) {
+            // do we have any evals left?
+            if( ! partial_eval_pair ) { // do we have an eval on either strand?
 
-				// yes - so evaluate the W-mer
-				static_cast< Derived * >( this )->evaluate_W_mer(
-					it,
-					partial_eval_pair,
-					bg_log_likelihood_bound,
-					num_occurrences
-				);
+                stats.discarded( w, num_occurrences ); // update stats
 
-			} else {
+            } else {
 
-				// can be over-ridden in derived class.
-				static_cast< Derived * >( this )->descend( it, partial_eval_pair, partial_bg_eval, w );
+                stats.evaluated( w, num_occurrences ); // update stats
 
-			}
-		}
+                // did we get to the bottom?
+                if( W == w ) {
+
+                    // yes - so evaluate the W-mer
+                    static_cast< Derived * >( this )->evaluate_W_mer(
+                        it,
+                        partial_eval_pair,
+                        bg_log_likelihood_bound,
+                        num_occurrences
+                    );
+
+                } else {
+
+                    // can be over-ridden in derived class.
+                    static_cast< Derived * >( this )->descend( it, partial_eval_pair, partial_bg_eval, w );
+
+                }
+            }
+        }
 	}
 
 
@@ -379,19 +390,19 @@ struct tree_descender
 		const double lambda = _model._prior.lambda.get_lambda();
 		const double log_lambda = _model._prior.lambda.get_log_lambda();
 
-		// calculate likelihoods of X_n given binding model
-		double log_p_X_n_theta_BS_positive, odds_ratio_positive;
+		// calculate likelihoods of X_n given binding model and log Bayes factors
+		double log_p_X_n_theta_BS_positive, lbf_positive;
 		if( bs_eval.first ) {
 			log_p_X_n_theta_BS_positive = _model.bs.eval_to_log( *bs_eval.first );
             if( ! _model.bg.have_base_pair_LLs() ) {
-                odds_ratio_positive = std::exp( log_p_background - log_p_X_n_theta_BS_positive );
+                lbf_positive = log_p_X_n_theta_BS_positive - log_p_background;
             }
 		}
-		double log_p_X_n_theta_BS_negative, odds_ratio_negative;
+		double log_p_X_n_theta_BS_negative, lbf_negative;
 		if( bs_eval.second ) {
 			log_p_X_n_theta_BS_negative = _model.bs.eval_to_log( *bs_eval.second );
 			if( ! _model.bg.have_base_pair_LLs() ) {
-			    odds_ratio_negative = std::exp( log_p_background - log_p_X_n_theta_BS_negative );
+			    lbf_negative = log_p_X_n_theta_BS_negative - log_p_background;
 			}
 		}
 
@@ -413,22 +424,20 @@ struct tree_descender
 
 			// prior
 			const double g_n = _model._prior._psp->get_g( occ.i1, occ.i2 );
-
-			// prior term
-			const double prior_term = ( 1. / ( lambda * ( 1. - g_n ) )  - 1. );
+			const double log_prior = _model._prior.calculate_log_prior( g_n );
 
 			// Z for the positive strand.
 			if( bs_eval.first ) {
 			    // if we are using a different LL for each occurrence
 	            if( _model.bg.have_base_pair_LLs() ) {
-	                odds_ratio_positive = std::exp( _model.bg.wmer_log_likelihood( occ.i1, occ.i2 ) - log_p_X_n_theta_BS_positive );
+	                lbf_positive = log_p_X_n_theta_BS_positive - _model.bg.wmer_log_likelihood( occ.i1, occ.i2 );
 	            }
-				const double Z_positive = 1. / ( 1. + prior_term * odds_ratio_positive );
-				MS_CHECK_BOUNDS( Z_positive, "Z_positive", 0., 1. );
-				Z_positive_total += Z_positive;
+				const double Z = lr_to_prob( std::exp( lbf_positive + log_prior ) );
+				MS_CHECK_BOUNDS( Z, "Z_positive", 0., 1. );
+				Z_positive_total += Z;
 				static_cast< Derived * >( this )->template handle_W_mer_occurrence_one_strand< false >(
 					global_pos,
-					Z_positive,
+					Z,
 					g_n,
 					lambda,
 					log_lambda,
@@ -441,14 +450,14 @@ struct tree_descender
 			if( bs_eval.second ) {
                 // if we are using a different LL for each occurrence
                 if( _model.bg.have_base_pair_LLs() ) {
-                    odds_ratio_negative = std::exp( _model.bg.wmer_log_likelihood( occ.i1, occ.i2 ) - log_p_X_n_theta_BS_negative );
+                    lbf_negative = log_p_X_n_theta_BS_negative - _model.bg.wmer_log_likelihood( occ.i1, occ.i2 );
                 }
-				const double Z_negative = 1. / ( 1. + prior_term * odds_ratio_negative );
-				MS_CHECK_BOUNDS( Z_negative, "Z_negative", 0., 1. );
-				Z_negative_total += Z_negative;
+                const double Z = lr_to_prob( std::exp( lbf_negative + log_prior ) );
+				MS_CHECK_BOUNDS( Z, "Z_negative", 0., 1. );
+				Z_negative_total += Z;
 				static_cast< Derived * >( this )->template handle_W_mer_occurrence_one_strand< true >(
 					global_pos,
-					Z_negative,
+					Z,
 					g_n,
 					lambda,
 					log_lambda,
